
class FormData(dict):
    ''' A dict that can be built from a form or formset instance, and will fill
    itself with request.POST-like data, allowing easier testing of form submissions.

    See forms_inline.TestTranslationsInline for example uses.
    '''
    def __init__(self, form_or_set):
        if hasattr(form_or_set, 'forms'):
            # It is a formset
            self.update(FormData(form_or_set.management_form))
            for form in form_or_set:
                self.update(FormData(form))
        else:
            # It is a form
            for field in form_or_set:
                value = field.value()
                initial = form_or_set.initial.get(field.name, field.field.initial)
                if value is not None:
                    self[field.html_name] = value
                if initial is not None:
                    self[field.html_initial_name] = initial

    def set_form_field(self, form, name, value):
        key = form[name].html_name
        if value is None:
            self.pop(key, None)
        else:
            self[key] = value

    def set_formset_field(self, formset, index, name, value):
        key = formset[index][name].html_name
        if value is None:
            self.pop(key, None)
        else:
            self[key] = value



# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from hvad.test_utils.data import NORMAL, STANDARD, CONCRETEAB, DATE, QONORMAL
from hvad.test_utils.project.app.models import Normal, Standard, ConcreteAB, Date, QONormal


class Fixture(object):
    translations = ('en', 'ja')
    def create_fixtures(self):
        pass

#===============================================================================

class NormalFixture(Fixture):
    normal_count = 0

    def create_fixtures(self):
        super(NormalFixture, self).create_fixtures()
        assert self.normal_count <= len(NORMAL), 'Not enough fixtures in data'

        self.normal_id = {}
        for i in range(1, self.normal_count + 1):
            self.normal_id[i] = self.create_normal(NORMAL[i]).pk

    def create_normal(self, data, translations=None):
        obj = Normal(shared_field=data.shared_field)
        for code in translations or self.translations:
            obj.translate(code)
            obj.translated_field = data.translated_field[code]
            obj.save()
        return obj


class StandardFixture(NormalFixture):
    standard_count = 0

    def create_fixtures(self):
        super(StandardFixture, self).create_fixtures()
        assert self.standard_count <= len(STANDARD)

        self.standard_id = {}
        for i in range(1, self.standard_count + 1):
            self.standard_id[i] = self.create_standard(STANDARD[i]).pk

    def create_standard(self, data):
        obj = Standard.objects.create(
            normal_field=data.normal_field,
            normal_id=self.normal_id[data.normal],
        )
        return obj


class QONormalFixture(Fixture):
    qonormal_count = 0

    def create_fixtures(self):
        super(QONormalFixture, self).create_fixtures()
        assert self.qonormal_count <= len(QONORMAL), 'Not enough fixtures in data'

        self.qonormal_id = {}
        for i in range(1, self.qonormal_count + 1):
            self.qonormal_id[i] = self.create_qonormal(QONORMAL[i]).pk

    def create_qonormal(self, data, translations=None):
        obj = QONormal(shared_field=data.shared_field)
        for code in translations or self.translations:
            obj.translate(code)
            obj.translated_field = data.translated_field[code]
            obj.save()
        return obj


class ConcreteABFixture(NormalFixture):
    concreteab_count = 0

    def create_fixtures(self):
        super(ConcreteABFixture, self).create_fixtures()
        assert self.concreteab_count <= len(CONCRETEAB)

        self.concreteab_id = {}
        for i in range(1, self.concreteab_count + 1):
            self.concreteab_id[i] = self.create_concreteab(CONCRETEAB[i]).pk

    def create_concreteab(self, data, translations=None):
        obj = ConcreteAB(
            shared_field_a=data.shared_field_a,
            shared_field_b_id=self.normal_id[data.shared_field_b],
            shared_field_ab=data.shared_field_ab,
        )
        for code in translations or self.translations:
            obj.translate(code)
            obj.translated_field_a_id = self.normal_id[data.translated_field_a[code]]
            obj.translated_field_b = data.translated_field_b[code]
            obj.translated_field_ab = data.translated_field_ab[code]
            obj.save()
        return obj


class DateFixture(Fixture):
    date_count = 0

    def create_fixtures(self):
        super(DateFixture, self).create_fixtures()
        assert self.date_count <= len(DATE)

        self.date_id = {}
        for i in range(1, self.date_count + 1):
            self.date_id[i] = self.create_date(DATE[i]).pk

    def create_date(self, data, translations=None):
        obj = Date(shared_date=data.shared_date)
        for code in translations or self.translations:
            obj.translate(code)
            obj.translated_date = data.translated_date[code]
            obj.save()
        return obj


class SuperuserFixture(Fixture):
    def create_fixtures(self):
        super(SuperuserFixture, self).create_fixtures()

        su = User(
            email='admin@admin.com',
            is_staff=True,
            is_superuser=True,
            is_active=True,
            username='admin',
        )
        su.set_password('admin')
        su.save()
        self.superuser = su



import django
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from django.db import models
from django.db.models.base import ModelBase
from django.db.models.fields import FieldDoesNotExist
from django.db.models.manager import Manager
from django.db.models.signals import post_save, class_prepared
from django.utils.translation import get_language
from hvad.descriptors import LanguageCodeAttribute, TranslatedAttribute
from hvad.manager import TranslationManager, TranslationsModelManager
from hvad.utils import (get_cached_translation, set_cached_translation,
                        SmartGetFieldByName, SmartGetField, settings_updater)
from hvad.compat import MethodType
import sys
import warnings

#===============================================================================

# Global settings, wrapped so they react to SettingsOverride
@settings_updater
def update_settings(*args, **kwargs):
    global FALLBACK_LANGUAGES, TABLE_NAME_SEPARATOR
    FALLBACK_LANGUAGES = tuple( code for code, name in settings.LANGUAGES )
    TABLE_NAME_SEPARATOR = getattr(settings, 'HVAD_TABLE_NAME_SEPARATOR', '_')

    if hasattr(settings, 'NANI_TABLE_NAME_SEPARATOR'):
        # remove in 1.3
        raise ImproperlyConfigured(
            'NANI_TABLE_NAME_SEPARATOR setting is obsolete and has been '
            'removed. Please rename it to HVAD_TABLE_NAME_SEPARATOR.')


#===============================================================================

def _split_together(constraints, fields, meta, name):
    sconst, tconst = [], []
    if name in meta:
        # raise in 1.3, remove in 1.5
        warnings.warn('Passing \'%s\' to TranslatedFields is deprecated. Please use '
                      'Please Meta.%s instead.' % (name, name), DeprecationWarning)
        tconst.extend(meta[name])

    for constraint in constraints:
        if all(item in fields for item in constraint):
            tconst.append(constraint)
        elif not any(item in fields for item in constraint):
            sconst.append(constraint)
        else:
            raise ImproperlyConfigured(
                'Constraints in Meta.%s cannot mix translated and '
                'untranslated fields, such as %r.' % (name, constraint))
    return sconst, tconst

def create_translations_model(model, related_name, meta, **fields):
    """
    Create the translations model for the shared model 'model'.
    'related_name' is the related name for the reverse FK from the translations
    model.
    'meta' is a (optional) dictionary of attributes for the translations model's
    inner Meta class.
    'fields' is a dictionary of fields to put on the translations model.
    
    Two fields are enforced on the translations model:
    
        language_code: A 15 char, db indexed field.
        master: A ForeignKey back to the shared model.
        
    Those two fields are unique together, this get's enforced in the inner Meta
    class of the translations table
    """

    # Build a list of translation models from base classes. Depth-first scan.
    abstract = model._meta.abstract
    translation_bases = []
    scan_bases = list(reversed(model.__bases__)) # backwards so we can use pop/extend
    while scan_bases:
        base = scan_bases.pop()
        if not issubclass(base, TranslatableModel) or base is TranslatableModel:
            continue
        try:
            # The base may have translations model, then just inherit that
            translation_bases.append(base._meta.translations_model)
        except AttributeError:
            # But it may not, and simply inherit other abstract bases, scan them
            scan_bases.extend(reversed(base.__bases__))
    translation_bases.append(BaseTranslationModel)

    # Create translation model Meta
    meta = meta or {}
    meta['abstract'] = abstract
    meta['db_tablespace'] = model._meta.db_tablespace
    meta['managed'] = model._meta.managed
    if model._meta.order_with_respect_to in fields:
        raise ImproperlyConfigured(
            'Using a translated fields in %s.Meta.order_with_respect_to is ambiguous '
            'and hvad does not support it.' %
            model._meta.model_name if django.VERSION >= (1, 6) else model._meta.module_name)

    sconst, tconst = _split_together(model._meta.unique_together, fields, meta, 'unique_together')
    model._meta.unique_together = tuple(sconst)
    meta['unique_together'] = tuple(tconst)
    if django.VERSION >= (1, 5):
        sconst, tconst = _split_together(model._meta.index_together, fields, meta, 'index_together')
        model._meta.index_together = tuple(sconst)
        meta['index_together'] = tuple(tconst)

    if not abstract:
        unique = [('language_code', 'master')]
        meta['unique_together'] = list(meta.get('unique_together', [])) + unique
    Meta = type('Meta', (object,), meta)

    if not hasattr(Meta, 'db_table'):
        Meta.db_table = model._meta.db_table + '%stranslation' % TABLE_NAME_SEPARATOR
    Meta.app_label = model._meta.app_label
    name = '%sTranslation' % model.__name__

    # Create translation model
    attrs = {}
    attrs.update(fields)
    attrs['Meta'] = Meta
    attrs['__module__'] = model.__module__

    if not abstract:
        # If this class is abstract, we must not contribute management fields
        attrs['objects'] = TranslationsModelManager()
        attrs['language_code'] = models.CharField(max_length=15, db_index=True)
        # null=True is so we can prevent cascade deletion
        attrs['master'] = models.ForeignKey(model, related_name=related_name,
                                            editable=False, null=True)
    # Create and return the new model
    translations_model = ModelBase(name, tuple(translation_bases), attrs)
    if not abstract:
        # Abstract models do not have a DNE class
        bases = (model.DoesNotExist, translations_model.DoesNotExist,)
        DNE = type('DoesNotExist', bases, {})
        translations_model.DoesNotExist = DNE
    opts = translations_model._meta
    opts.shared_model = model

    # We need to set it here so it is available when we scan subclasses
    model._meta.translations_model = translations_model

    # Register it as a global in the shared model's module.
    # This is needed so that Translation model instances, and objects which
    # refer to them, can be properly pickled and unpickled. The Django session
    # and caching frameworks, in particular, depend on this behaviour.
    mod = sys.modules[model.__module__]
    setattr(mod, name, translations_model)

    return translations_model


class TranslatedFields(object):
    """
    Wrapper class to define translated fields on a model.
    """
    def __init__(self, meta=None, **fields):
        self.fields = fields
        self.meta = meta

    def contribute_to_class(self, cls, name):
        """
        Called from django.db.models.base.ModelBase.__new__
        """
        create_translations_model(cls, name, self.meta, **self.fields)


class BaseTranslationModel(models.Model):
    """
    Needed for detection of translation models. Due to the way dynamic classes
    are created, we cannot put the 'language_code' field on here.
    """
    class Meta:
        abstract = True


class TranslatableModelBase(ModelBase):
    def __new__(cls, *args, **kwargs):
        # remove in 1.3
        raise RuntimeError(
            'TranslatableModelBase metaclass is no longer used and has been '
            'removed. Hvad no longer uses a custom metaclass so conflict '
            'resolution is no longer required, TranslatableModelBase can be '
            'dropped.')


class NoTranslation(object):
    pass


class TranslatableModel(models.Model):
    """
    Base model for all models supporting translated fields (via TranslatedFields).
    """
    # change the default manager to the translation manager
    objects = TranslationManager()

    class Meta:
        abstract = True

    def __init__(self, *args, **kwargs):
        # Split arguments into shared/translatd
        veto_names = ('pk', 'master', 'master_id', self._meta.translations_model._meta.pk.name)
        skwargs, tkwargs = {}, {}
        for key, value in kwargs.items():
            if key in self._translated_field_names and not key in veto_names:
                tkwargs[key] = value
            else:
                skwargs[key] = value

        super(TranslatableModel, self).__init__(*args, **skwargs)

        # Create a translation if there are translated fields
        if tkwargs:
            tkwargs['language_code'] = tkwargs.get('language_code') or get_language()
            set_cached_translation(self, self._meta.translations_model(**tkwargs))

    @classmethod
    def save_translations(cls, instance, **kwargs):
        'Signal handler for post_save'
        translation = get_cached_translation(instance)
        if translation is not None:
            translation.master = instance
            translation.save()

    def translate(self, language_code):
        ''' Create a new translation for current instance.
            Does NOT check if the translation already exists!
        '''
        set_cached_translation(
            self,
            self._meta.translations_model(language_code=language_code)
        )
        return self

    def safe_translation_getter(self, name, default=None):
        cache = get_cached_translation(self)
        if cache is None:
            return default
        return getattr(cache, name, default)

    def lazy_translation_getter(self, name, default=None):
        """
        Lazy translation getter that fetches translations from DB in case the instance is currently untranslated and
        saves the translation instance in the translation cache
        """
        stuff = self.safe_translation_getter(name, NoTranslation)
        if stuff is not NoTranslation:
            return stuff

        # get all translations
        translations = getattr(self, self._meta.translations_accessor).all()

        # if no translation exists, bail out now
        if len(translations) == 0:
            return default

        # organize translations into a nice dict
        translation_dict = dict((t.language_code, t) for t in translations)

        # see if we have the right language, or any language in fallbacks
        for code in (get_language(), settings.LANGUAGE_CODE) + FALLBACK_LANGUAGES:
            try:
                translation = translation_dict[code]
            except KeyError:
                continue
            break
        else:
            # none of the fallbacks was found, pick an arbitrary translation
            translation = translation_dict.popitem()[1]

        set_cached_translation(self, translation)
        return getattr(translation, name, default)

    def get_available_languages(self):
        """ Get a list of all available language_code in db. """
        qs = getattr(self, self._meta.translations_accessor).all()
        if qs._result_cache is not None:
            return [obj.language_code for obj in qs]
        return qs.values_list('language_code', flat=True)
    
    #===========================================================================
    # Internals
    #===========================================================================
    
    @property
    def _translated_field_names(self):
        if getattr(self, '_translated_field_names_cache', None) is None:
            opts = self._meta.translations_model._meta
            result = set()

            if django.VERSION >= (1, 8):
                for field in opts.get_fields():
                    result.add(field.name)
                    if hasattr(field, 'attname'):
                        result.add(field.attname)
            else:
                result = set(opts.get_all_field_names())
                for name in tuple(result):
                    try:
                        attname = opts.get_field(name).get_attname()
                    except (FieldDoesNotExist, AttributeError):
                        continue
                    if attname:
                        result.add(attname)

            self._translated_field_names_cache = tuple(result)
        return self._translated_field_names_cache

#=============================================================================

def contribute_translations(cls, rel):
    """
    Contribute translations options to the inner Meta class and set the
    descriptors.

    This get's called from prepare_translatable_model
    """
    opts = cls._meta
    opts.translations_accessor = rel.get_accessor_name()
    if django.VERSION >= (1, 8):
        opts.translations_model = rel.field.model
    else:
        opts.translations_model = rel.model
    opts.translations_cache = '%s_cache' % rel.get_accessor_name()
    trans_opts = opts.translations_model._meta

    # Set descriptors
    ignore_fields = ('pk', 'master', 'master_id', opts.translations_model._meta.pk.name)
    for field in trans_opts.fields:
        if field.name in ignore_fields:
            continue
        if field.name == 'language_code':
            attr = LanguageCodeAttribute(opts)
        else:
            attr = TranslatedAttribute(opts, field.name)
            attname = field.get_attname()
            if attname and attname != field.name:
                setattr(cls, attname, TranslatedAttribute(opts, attname))
        setattr(cls, field.name, attr)


def prepare_translatable_model(sender, **kwargs):
    model = sender
    if not issubclass(model, TranslatableModel) or model._meta.abstract:
        return
    if not isinstance(model._default_manager, TranslationManager):
        raise ImproperlyConfigured(
            "The default manager on a TranslatableModel must be a "
            "TranslationManager instance or an instance of a subclass of "
            "TranslationManager, the default manager of %r is not." % model)

    # If this is a proxy model, get the concrete one
    concrete_model = model._meta.concrete_model if model._meta.proxy else model

    # Find the instance of TranslatedFields in the concrete model's dict
    # We cannot use _meta.get_fields here as app registry is not ready yet.
    found = None
    for relation in list(concrete_model.__dict__.keys()):
        try:
            obj = getattr(model, relation)
            if django.VERSION >= (1, 8):
                shared_model = obj.related.field.model._meta.shared_model
            else:
                shared_model = obj.related.model._meta.shared_model
        except AttributeError:
            continue
        if shared_model is concrete_model:
            if found:
                raise ImproperlyConfigured(
                    "A TranslatableModel can only define one set of "
                    "TranslatedFields, %r defines more than one: %r on %r "
                    "and %r on %r and possibly more" % (model, obj,
                    obj.related.model, found, found.related.model))
            # Mark as found but keep looking so we catch duplicates and raise
            found = obj

    if not found:
        raise ImproperlyConfigured(
            "No TranslatedFields found on %r, subclasses of "
            "TranslatableModel must define TranslatedFields." % model
        )

    #### Now we have to work ####

    contribute_translations(model, found.related)

    # Ensure _base_manager cannot be TranslationManager despite use_for_related_fields
    # 1- it is useless unless default_class is overriden
    # 2- in that case, _base_manager is used for saving objects and must not be
    #    translation aware.
    base_mgr = getattr(model, '_base_manager', None)
    if base_mgr is None or isinstance(base_mgr, TranslationManager):
        model.add_to_class('_base_manager', Manager())

    # Replace get_field_by_name with one that warns for common mistakes
    if django.VERSION < (1, 9) and not isinstance(model._meta.get_field_by_name, SmartGetFieldByName):
        model._meta.get_field_by_name = MethodType(
            SmartGetFieldByName(model._meta.get_field_by_name),
            model._meta
        )
    if not isinstance(model._meta.get_field, SmartGetField):
        model._meta.get_field = MethodType(
            SmartGetField(model._meta.get_field),
            model._meta
        )

    # Attach save_translations
    post_save.connect(model.save_translations, sender=model, weak=False)

class_prepared.connect(prepare_translatable_model)



import django
from django.db.models.fields import FieldDoesNotExist
from django.test.signals import setting_changed
from django.utils.translation import get_language
from hvad.exceptions import WrongManager
import warnings

#=============================================================================
# Translation manipulators

def get_cached_translation(instance):
    'Get currently cached translation of the instance'
    return getattr(instance, instance._meta.translations_cache, None)

def set_cached_translation(instance, translation):
    '''Sets the translation cached onto instance.
        - Passing None unsets the translation cache
        - Returns the translation that was loaded before
    '''
    tcache = instance._meta.translations_cache
    previous = getattr(instance, tcache, None)
    if translation is None:
        if previous is not None:
            delattr(instance, tcache)
    else:
        setattr(instance, tcache, translation)
    return previous

def combine(trans, klass):
    """
    'Combine' the shared and translated instances by setting the translation
    on the 'translations_cache' attribute of the shared instance and returning
    the shared instance.

    The result is casted to klass (needed for proxy models).
    """
    combined = trans.master
    if klass._meta.proxy:
        combined.__class__ = klass
    setattr(combined, combined._meta.translations_cache, trans)
    return combined


def get_translation(instance, language_code=None):
    ''' Get translation by language. Fresh copy is loaded from DB.
        Can leverage prefetched data, like in .prefetch_related('translations')
    '''
    accessor = getattr(instance, instance._meta.translations_accessor)

    language_code = language_code or get_language()
    qs = accessor.all()
    if qs._result_cache is None:
        return accessor.get(language_code=language_code)
    else: # take advantage of cached translations
        for obj in qs:
            if obj.language_code == language_code:
                return obj
    raise accessor.model.DoesNotExist

def load_translation(instance, language, enforce=False):
    ''' Get or create a translation.
        Depending on enforce argument, the language will serve as a default
        or will be enforced by reloading a mismatching translation.

        If instance's active translation is in given language, this is
        a guaranteed no-op: it will be returned as is.
    '''
    trans_model = instance._meta.translations_model
    translation = get_cached_translation(instance)

    if translation is None or (enforce and translation.language_code != language):
        if instance.pk is None:
            translation = trans_model(language_code=language)
        else:
            try:
                translation = get_translation(instance, language)
            except trans_model.DoesNotExist:
                translation = trans_model(language_code=language)
    return translation

#=============================================================================

def get_translation_aware_manager(model):
    from hvad.manager import TranslationAwareManager
    manager = TranslationAwareManager()
    manager.model = model
    return manager

# remove when we drop support for django 1.8
class SmartGetFieldByName(object):
    """
    Get field by name from a shared model or raise a smart exception to help the
    developer.
    """
    def __init__(self, real):
        self.real = real
    
    def __call__(self, meta, name):
        assert not isinstance(self.real, SmartGetFieldByName)
        try:
            return self.real(name)
        except FieldDoesNotExist as e:
            try:
                meta.translations_model._meta.get_field_by_name(name)
            except FieldDoesNotExist:
                raise e
            else:
                raise WrongManager("To access translated fields like %r from "
                                   "an untranslated model, you must use a "
                                   "translation aware manager. For non-translatable "
                                   "models, you can get one using "
                                   "hvad.utils.get_translation_aware_manager.\n"
                                   "For translatable models, use the language() "
                                   "method."%
                                   name)


class SmartGetField(object):
    ''' Smart get_field that raises a helpful exception on get_field() '''
    def __init__(self, real):
        assert not isinstance(real, SmartGetField)
        self.real = real

    def __call__(self, meta, name):
        try:
            return self.real(name)
        except FieldDoesNotExist as e:
            try:
                meta.translations_model._meta.get_field(name)
            except FieldDoesNotExist:
                raise e
            else:
                raise WrongManager("To access translated fields like %r from "
                                   "an untranslated model, you must use a "
                                   "translation aware manager. For non-translatable "
                                   "models, you can get one using "
                                   "hvad.utils.get_translation_aware_manager.\n"
                                   "For translatable models, use the language() "
                                   "method." % name)

def collect_context_modifiers(instance, include=None, exclude=None, extra_kwargs=None):
    """
    helper method that updates the context with any instance methods that start
    with `context_modifier_`. `include` is an optional list of method names
    that also should be called. Any method names in `exclude` will not be
    added to the context.

    This helper is most useful when called from get_context_data()::

        def get_context_data(self, **kwargs):
            context = super(MyViewClass, self).get_context_data(**kwargs)
            context.update(collect_context_modifiers(self, extra_kwargs=kwargs))
            return context
    """
    include = include or []
    exclude = exclude or []
    extra_kwargs = extra_kwargs or {}
    context = {}

    for thing in dir(instance):
        if (thing.startswith('context_modifier_') or thing in include) and \
            not thing in exclude:
            context.update(getattr(instance, thing, lambda x:x)(**extra_kwargs))

    if context:
        # raise in 1.3, remove whole function in 1.5
        warnings.warn('Context modifiers are deprecated and will be removed '
                      'in hvad version 1.3. Triggering class: %r' % instance.__class__,
                      DeprecationWarning)
    return context

#=============================================================================
# Internal sugar

class _MinimumDjangoVersionDescriptor(object):
    ''' Ensures methods that do not exist on current Django version raise a
        helpful message and an actual AttributeError
    '''
    def __init__(self, name, version):
        self.name = name
        self.version = version

    def __get__(self, obj, type=None):
        raise AttributeError('Method %s requires Django %s or newer' %
                             (self.name, '.'.join(str(x) for x in self.version)))

def minimumDjangoVersion(*args):
    ''' Method/attribute decorator making it unavailable on older Django versions
        e.g.: @minimumDjangoVersion(1, 4, 2)
    '''
    if django.VERSION >= args:
        return lambda x: x
    return lambda x: _MinimumDjangoVersionDescriptor(x.__name__, args)

def settings_updater(func):
    ''' Decorator for setting globals depending on django settings '''
    func()
    setting_changed.connect(func, dispatch_uid=id(func))
    return func



from django.views.generic.detail import SingleObjectMixin, SingleObjectTemplateResponseMixin
from django.views.generic.edit import ModelFormMixin, ProcessFormView, BaseDeleteView
from django.utils.translation import get_language
from .forms import translatable_modelform_factory
from .utils import collect_context_modifiers
import warnings

class _TransitionObjectMixin(SingleObjectMixin):
    def get_object(self, queryset=None):
        _get_object = getattr(self, '_get_object', None)
        if callable(_get_object):
            # raise in 1.3, remove in 1.5
            warnings.warn('Method \'_get_object()\' is deprecated and will be removed. '
                          'Please update view %s to use \'get_object()\' instead.'
                          % self.__class__.__name__, DeprecationWarning)
            return _get_object(queryset)

        filter_kwargs = getattr(self, 'filter_kwargs', None)
        if callable(filter_kwargs):
            # raise in 1.3, remove in 1.5
            warnings.warn('Method \'filter_kwargs()\' is deprecated and will be removed. '
                          'Please update view %s to use \'get_queryset()\' or '
                          '\'get_object()\'.' % self.__class__.__name__,
                          DeprecationWarning)
            if queryset is None:
                queryset = self.get_queryset()
            return queryset.get(**filter_kwargs())

        elif (self.pk_url_kwarg == 'pk' and 'object_id' in self.kwargs and 'pk' not in self.kwargs):
            # raise in 1.3, remove in 1.5
            warnings.warn('Default view argument for pk has changed from \'object_id\' '
                          'to \'pk\'. Please update view %s.' % self.__class__.__name__,
                          DeprecationWarning)
            self.kwargs['pk'] = self.kwargs['object_id']

        return super(_TransitionObjectMixin, self).get_object(queryset)


class TranslatableModelFormMixin(ModelFormMixin, _TransitionObjectMixin):
    ''' ModelFormMixin that works with an TranslatableModelForm in **enforce** mode '''
    query_language_key = 'language'

    def get_language(self):
        legacy = getattr(self, '_language', None)
        if callable(legacy):
            # raise in 1.3, remove in 1.5
            warnings.warn('Method \'_language\' has been renamed to \'get_language()\'. '
                          'Please update view %s.' % self.__class__.__name__,
                          DeprecationWarning)
            return legacy(self.request)
        return self.request.GET.get(self.query_language_key) or get_language()

    def get_form_class(self):
        if self.model is not None:
            model = self.model
        elif getattr(self, 'object', None) is not None:
            model = self.object.__class__
        else:
            qs = self.get_queryset()
            model = getattr(qs, 'shared_model', qs.model)

        kwargs = {}
        if self.form_class is not None:
            kwargs['form'] = self.form_class
        return translatable_modelform_factory(self.get_language(), model, **kwargs)

    def get_context_data(self, **kwargs):
        # Deprecation warning is triggered inside collect_context_modifiers
        # remove this in 1.5
        context = super(TranslatableModelFormMixin, self).get_context_data(**kwargs)
        context.update(collect_context_modifiers(self, extra_kwargs=kwargs))
        return context

#=============================================================================

class TranslatableBaseCreateView(TranslatableModelFormMixin, ProcessFormView):
    def get(self, request, *args, **kwargs):
        self.object = None
        return super(TranslatableBaseCreateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        return super(TranslatableBaseCreateView, self).post(request, *args, **kwargs)

class TranslatableCreateView(SingleObjectTemplateResponseMixin, TranslatableBaseCreateView):
    template_name_suffix = '_form'

#-------------------------------------------------------------------------

class TranslatableBaseUpdateView(TranslatableModelFormMixin, ProcessFormView):
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(TranslatableBaseUpdateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(TranslatableBaseUpdateView, self).post(request, *args, **kwargs)

class TranslatableUpdateView(SingleObjectTemplateResponseMixin, TranslatableBaseUpdateView):
    template_name_suffix = '_form'

#-------------------------------------------------------------------------

class TranslatableBaseDeleteView(BaseDeleteView, _TransitionObjectMixin):
    pass

class TranslatableDeleteView(SingleObjectTemplateResponseMixin, TranslatableBaseDeleteView):
    template_name_suffix = '_confirm_delete'

#=============================================================================
#=============================================================================
#=============================================================================

from django.views.generic.edit import UpdateView
from .admin import TranslatableModelAdminMixin
from .forms import TranslatableModelForm

class TranslatableBaseView(UpdateView, TranslatableModelAdminMixin): #pragma: no cover
    form_class = TranslatableModelForm

    def __init__(self, *args, **kwargs):
        warnings.warn('TranslatableBaseView is deprecated and will be removed '
                      'in release 1.3. Please update %s to use new Django-compliant '
                      'views instead.' % self.__class__.__name__,
                      DeprecationWarning, stacklevel=2)
        super(TranslatableBaseView, self).__init__(*args, **kwargs)

    def filter_kwargs(self):
        if "slug" in self.kwargs:
            return {self.slug_field: self.kwargs["slug"]}
        return {'pk': self.kwargs['object_id']}

    def get_form_class(self):
        language = self._language(self.request)
        return translatable_modelform_factory(language, self.model, form=self.form_class)

    def get_queryset(self):
        if self.queryset is None:
            if self.model:
                language = self._language(self.request)
                return self.model._default_manager.language(language)

    def _get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()
        model = self.model
        try:
            obj = queryset.get(**self.filter_kwargs())
        except self.model.DoesNotExist:
            obj = None
        if obj:
            return obj
        queryset = self.model._default_manager.untranslated()
        try:
            obj = queryset.get(**self.filter_kwargs())
        except model.DoesNotExist:
            return None
        obj.translate(self._language(self.request))
        return obj

    def context_modifier_languages_available(self, **kwargs):
        context = {
            'language_tabs': self.get_language_tabs(self.request, self.get_available_languages(self.object))
        }
        return context

    def get_context_data(self, **kwargs):
        context = super(TranslatableBaseView, self).get_context_data(**kwargs)
        context.update(collect_context_modifiers(self, extra_kwargs=kwargs))
        return context




from django.contrib import admin
from hvad.test_utils.project.app.models import Normal, SimpleRelated, LimitedChoice, AutoPopulated
from hvad.admin import TranslatableAdmin, TranslatableTabularInline

class SimpleRelatedInline(TranslatableTabularInline):
    model = SimpleRelated

class NormalAdmin(TranslatableAdmin):
    inlines = [SimpleRelatedInline,]

admin.site.register(Normal, NormalAdmin)
admin.site.register(SimpleRelated, TranslatableAdmin)
admin.site.register(LimitedChoice)
admin.site.register(AutoPopulated, TranslatableAdmin)




# Sphinx helper for Django-specific references

def setup(app):
    app.add_crossref_type(
        directivename = "label",
        rolename = "djterm",
        indextemplate = "pair: %s; label",
    )
    app.add_crossref_type(
        directivename = "setting",
        rolename = "setting",
        indextemplate = "pair: %s; setting",
    )
    app.add_crossref_type(
        directivename = "templatetag",
        rolename = "ttag",
        indextemplate = "pair: %s; template tag",
    )
    app.add_crossref_type(
        directivename = "templatefilter",
        rolename = "tfilter",
        indextemplate = "pair: %s; template filter",
    )
    app.add_crossref_type(
        directivename = "fieldlookup",
        rolename = "lookup",
        indextemplate = "pair: %s; field lookup type",
    )



import django
from django.db import models
from django.template.defaultfilters import slugify
from hvad.models import TranslatableModel, TranslatedFields
from hvad.manager import TranslationManager, TranslationQueryset
from django.utils.encoding import python_2_unicode_compatible

#===============================================================================
# Basic models

@python_2_unicode_compatible
class Normal(TranslatableModel):
    shared_field = models.CharField(max_length=255)
    translations = TranslatedFields(
        translated_field = models.CharField(max_length=255)
    )

    def __str__(self):
        return self.safe_translation_getter('translated_field', self.shared_field)

@python_2_unicode_compatible
class Unique(TranslatableModel):
    shared_field = models.CharField(max_length=255, unique=True)
    translations = TranslatedFields(
        translated_field = models.CharField(max_length=255, unique=True)
    )

    def __str__(self):
        return self.safe_translation_getter('translated_field', self.shared_field)

@python_2_unicode_compatible
class NormalProxy(Normal):
    def __str__(self):
        return u'proxied %s' % super(NormalProxy, self).__str__()

    class Meta:
        proxy = True

@python_2_unicode_compatible
class NormalProxyProxy(NormalProxy):
    def __str__(self):
        return u'proxied^2 %s' % super(NormalProxyProxy, self).__str__()

    class Meta:
        proxy = True

#===============================================================================
# Models for testing relations

class Related(TranslatableModel):
    """ Model with foreign keys to Normal, both shared and translatable """
    normal = models.ForeignKey(Normal, related_name='rel1', null=True)
    translated_fields = TranslatedFields(
        translated = models.ForeignKey(Normal, related_name='rel3', null=True),
        translated_to_translated = models.ForeignKey(Normal, related_name='rel4', null=True),
    )

class RelatedProxy(Related):
    """ Proxy to model with foreign keys to Normal, both shared and translatable """
    class Meta:
        proxy = True


class SimpleRelated(TranslatableModel):
    """ Model with foreign key to Normal, shared only and regular translatable field """
    normal = models.ForeignKey(Normal, related_name='simplerel')
    manynormals = models.ManyToManyField(Normal, blank=True, related_name='manysimplerel')
    translated_fields = TranslatedFields(
        translated_field = models.CharField(max_length=255),
    )

class SimpleRelatedProxy(SimpleRelated):
    """ Proxy to model with foreign key to Normal, shared only and regular translatable field """
    class Meta:
        proxy = True


class RelatedRelated(TranslatableModel):
    """ Model with foreign keys to Related and SimpleRelated, both shared and transltable.
        This is used to test deep relations to Normal
    """
    related = models.ForeignKey(Related, related_name='+', null=True)
    simple = models.ForeignKey(SimpleRelated, related_name='+', null=True)

    translated_fields = TranslatedFields(
        trans_related = models.ForeignKey(Related, related_name='+', null=True),
        trans_simple = models.ForeignKey(SimpleRelated, related_name='+', null=True),
    )


@python_2_unicode_compatible
class Many(models.Model):
    """ Untranslatable Model with M2M key to Normal """
    name = models.CharField(max_length=128)
    normals = models.ManyToManyField(Normal, related_name="manyrels")

    def __str__(self):
        return self.name


class Standard(models.Model):
    """ Untranslatable Model with foreign key to Normal """
    normal_field = models.CharField(max_length=255)
    normal = models.ForeignKey(Normal, related_name='standards')
    date = models.ForeignKey('Date', null=True, related_name='standards')


class StandardRelated(TranslatableModel):
    """ Translatable mode with foreign key to untranslatable """
    shared_field = models.CharField(max_length=255)
    standard = models.ForeignKey(Standard, related_name='related')
    translations = TranslatedFields(
        translated_field = models.CharField(max_length=255),
    )


class FullTranslationManager(TranslationManager):
    use_for_related_fields = True
    default_class = TranslationQueryset

class QONormal(TranslatableModel):
    shared_field = models.CharField(max_length=255)
    translations = TranslatedFields(
        translated_field = models.CharField(max_length=255)
    )
    objects = FullTranslationManager()

class QOSimpleRelated(TranslatableModel):
    normal = models.ForeignKey(QONormal, related_name='simplerel', null=True)
    translations = TranslatedFields(
        translated_field = models.CharField(max_length=255),
    )
    objects = FullTranslationManager()

class QOMany(TranslatableModel):
    normals = models.ManyToManyField(QONormal, related_name="manyrels")
    translations = TranslatedFields(
        translated_field = models.CharField(max_length=255),
    )
    objects = FullTranslationManager()


#===============================================================================
# Models for testing abstract model support
#
# This creates an abstract hierarchy like this:
#
# AbstractA
#     |
# AbstractAA      AbstractB
#     \_______________/
#             |
#         ConcreteAB
#             |
#      ConcreteABProxy

class AbstractA(TranslatableModel):
    translations = TranslatedFields(
        translated_field_a = models.ForeignKey(Normal, related_name='%(class)s_set'),
    )
    class Meta:
        abstract = True

class AbstractAA(AbstractA):
    shared_field_a = models.CharField(max_length=255)
    class Meta:
        abstract = True

class AbstractB(TranslatableModel):
    shared_field_b = models.ForeignKey(Normal, related_name='%(class)s_set')
    translations = TranslatedFields(
        translated_field_b = models.CharField(max_length=255),
    )
    class Meta:
        abstract = True

@python_2_unicode_compatible
class ConcreteAB(AbstractAA, AbstractB):
    shared_field_ab = models.CharField(max_length=255)
    translations = TranslatedFields(
        translated_field_ab = models.CharField(max_length=255),
    )

    def __str__(self):
        return '%s, %s, %s' % (
            str(self.safe_translation_getter('translated_field_a', self.shared_field_a)),
            self.safe_translation_getter('translated_field_b', str(self.shared_field_b)),
            self.safe_translation_getter('translated_field_ab', self.shared_field_ab),
        )

@python_2_unicode_compatible
class ConcreteABProxy(ConcreteAB):
    def __str__(self):
        return 'proxied %s, %s, %s' % (
            str(self.safe_translation_getter('translated_field_a', self.shared_field_a)),
            self.safe_translation_getter('translated_field_b', str(self.shared_field_b)),
            self.safe_translation_getter('translated_field_ab', self.shared_field_ab),
        )
    class Meta:
        proxy = True

#===============================================================================
# Models for testing choice limiting in foreign keys

class LimitedChoice(models.Model):
    choice_fk = models.ForeignKey(Normal,
        limit_choices_to={'shared_field__startswith': 'Shared1',},
        related_name='limitedchoices_fk',
    )
    choice_mm = models.ManyToManyField(Normal,
        limit_choices_to={'shared_field__startswith': 'Shared2',},
        related_name='limitedchoices_mm',
    )

#===============================================================================
# Model for testing miscellaneous data and query types

class Date(TranslatableModel):
    """ Model for testing Date manipulation """
    shared_date = models.DateTimeField()
    translated_fields = TranslatedFields(
        translated_date = models.DateTimeField()
    )

    class Meta:
        get_latest_by = 'shared_date'

#===============================================================================

class AggregateModel(TranslatableModel):
    """ Model for testing queryset aggregation """
    number = models.IntegerField()
    translated_fields = TranslatedFields(
        translated_number = models.IntegerField(),
    )


class MultipleFields(TranslatableModel):
    """ Model for testing multi-field queries """
    first_shared_field = models.CharField(max_length=255)
    second_shared_field = models.CharField(max_length=255)
    translations = TranslatedFields(
        first_translated_field = models.CharField(max_length=255),
        second_translated_field = models.CharField(max_length=255)
    )


class Boolean(TranslatableModel):
    """ Model for testing boolean data manipulation """
    shared_flag = models.BooleanField(default=False)
    translations = TranslatedFields(
        translated_flag = models.BooleanField(default=False)
    )


class AutoPopulated(TranslatableModel):
    """ Model for testing custom save method """
    translations = TranslatedFields(
        slug = models.SlugField(max_length=255, blank=True),
        translated_name = models.CharField(max_length=255),
    )

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.translated_name[:125])
        super(AutoPopulated, self).save(*args, **kwargs)



import django
from django.db.models import Q, FieldDoesNotExist
from django.db.models.expressions import ExpressionNode
from django.db.models.sql.where import WhereNode
from collections import namedtuple

#===============================================================================
# Generators abstracting walking through internal django structures

QueryTerm = namedtuple('QueryTerm', 'depth term model field translated target many')

def query_terms(model, path):
    """ Yields QueryTerms of given path starting from given model.
        - model can be either a regular model or a translatable model
    """
    bits = path.split('__')
    field = None
    for depth, bit in enumerate(bits):
        # STEP 1 -- Resolve the field
        if bit == 'pk': # handle 'pk' alias
            bit = model._meta.pk.name

        try:
            if django.VERSION >= (1, 8):
                try:                        # is field on the shared model?
                    field = model._meta.get_field.real(bit)
                    translated = False
                except FieldDoesNotExist:   # nope, get field from translations model
                    field = model._meta.translations_model._meta.get_field(bit)
                    translated = True
                except AttributeError:      # current model is a standard model
                    field = model._meta.get_field(bit)
                    translated = False
                direct = (
                    not field.auto_created or
                    getattr(field, 'db_column', None) or
                    getattr(field, 'attname', None)
                )
            else:
                # older versions do not retrieve reverse/m2m with get_field, we must use the obsolete api
                try:
                    field, _, direct, _ = model._meta.get_field_by_name.real(bit)
                    translated = False
                except FieldDoesNotExist:
                    field, _, direct, _ = model._meta.translations_model._meta.get_field_by_name(bit)
                    translated = True
                except AttributeError:
                    field, _, direct, _ = model._meta.get_field_by_name(bit)
                    translated = False
        except FieldDoesNotExist:
            break


        # STEP 2 -- Find out the target of the relation, if it is one
        if direct:  # field is on model
            if field.rel:    # field is a foreign key, follow it
                target = field.rel.to._meta.concrete_model
            else:            # field is a regular field
                target = None
        else:       # field is a m2m or reverse fk, follow it
            target = (field.related_model._meta.concrete_model if django.VERSION >= (1, 8) else
                      field.model._meta.concrete_model)

        yield QueryTerm(
            depth=depth,
            term=bit,
            model=model,
            field=field,
            translated=translated,
            target=target,
            many=not direct
        )

        # Onto next iteration
        if target is None:
            depth += 1   # we hit a regular field, mark it as yielded then break
            break        # through to lookup/transform flushing
        model = target

    else:
        return  # all bits were recognized as fields, job done

    # STEP 3 -- Flush lookup/transform bits - do not handle invalid stuff, Django will do it
    for depth, bit in enumerate(bits[depth:], depth):
        yield QueryTerm(
            depth=depth,
            term=bit,
            model=model,
            field=None,
            translated=None,
            target=None,
            many=False
        )


def q_children(q):
    ''' Recursively visit a Q object, yielding each (key, value) pair found.
        - q: the Q object to visit
        - Yields a 3-tuple ((key, value), containing_list, index_in_list) so
          as to allow updating the tuple in the list
    '''
    todo = [q]
    while todo:
        q = todo.pop()
        for index, child in enumerate(q.children):
            if isinstance(child, Q):
                todo.append(child)
            else:
                yield child, q.children, index

if django.VERSION >= (1, 8):
    def expression_nodes(expression):
        ''' Recursively visit an expression object, yielding each node in turn.
            - expression: the expression object to visit
        '''
        todo = [expression]
        while todo:
            expression = todo.pop()
            if expression is not None:
                yield expression
            if isinstance(expression, ExpressionNode):
                todo.extend(expression.get_source_expressions())

else:
    def expression_nodes(expression):
        ''' Recursively visit an expression object, yielding each node in turn.
            - expression: the expression object to visit
        '''
        todo = [expression]
        while todo:
            expression = todo.pop()
            if expression is not None:
                yield expression
            if isinstance(expression, ExpressionNode):
                todo.extend(expression.children or ())


def where_node_children(node):
    ''' Recursively visit all children of a where node, yielding each field in turn.
        - node: the node to visit
    '''
    todo = [node]
    get_field_name = ((lambda n: n.lhs.target.name) if django.VERSION >= (1, 7) else
                      (lambda n: n[0].field.name))
    while todo:
        node = todo.pop()
        for child in node.children:
            try:
                field_name = get_field_name(child)
            except (TypeError, AttributeError):
                pass
            else:
                yield child, field_name
            if isinstance(child, WhereNode):
                todo.append(child)



# -*- coding: utf-8 -*-
from datetime import datetime
from collections import namedtuple

#===============================================================================

NormalData = namedtuple('NormalData', 'shared_field translated_field')

NORMAL = {
    1: NormalData(
        shared_field=u'Shared1',
        translated_field={'en': u'English1', 'ja': u'日本語一',},
    ),
    2: NormalData(
        shared_field=u'Shared2',
        translated_field={'en': u'English2', 'ja': u'日本語二',},
    ),
}

QONORMAL = NORMAL

#===============================================================================

StandardData = namedtuple('StandardData', 'normal_field normal')

STANDARD = {
    1: StandardData(normal_field=u'normal1', normal=1),
    2: StandardData(normal_field=u'normal2', normal=2),
}

#===============================================================================

ConcreteABData = namedtuple('ConcreteABData',
                            'shared_field_a shared_field_b shared_field_ab '
                            'translated_field_a translated_field_b translated_field_ab')

CONCRETEAB = {
    1: ConcreteABData(
        shared_field_a=u'SharedA1',
        shared_field_b=1,
        shared_field_ab=u'SharedAB1',
        translated_field_a={'en': 1, 'ja': 2,},
        translated_field_b={'en': u'EnglishB1', 'ja': u'日本語一',},
        translated_field_ab={'en': u'EnglishAB1', 'ja': u'日本語一',},
    ),
    2: ConcreteABData(
        shared_field_a=u'SharedA2',
        shared_field_b=2,
        shared_field_ab=u'SharedAB2',
        translated_field_a={'en': 2, 'ja': 1,},
        translated_field_b={'en': u'EnglishB2', 'ja': u'日本語二',},
        translated_field_ab={'en': u'EnglishAB2', 'ja': u'日本語二',},
    ),
}

#===============================================================================

DateData = namedtuple('DateData', 'shared_date translated_date')

DATE_VALUES = (
    # Values here must be in ascending order so earliest/latest tests work
    datetime(year=1988, month=7, day=4),
    datetime(year=2011, month=1, day=26),
    datetime(year=2011, month=2, day=16, hour=5, minute=1, second=14),
)

DATE = {
    # Be sure to update DATE_REVERSED when modifying this
    1: DateData(shared_date=DATE_VALUES[0],
                translated_date={'en': DATE_VALUES[2], 'ja': DATE_VALUES[1],},),
    2: DateData(shared_date=DATE_VALUES[2],
                translated_date={'en': DATE_VALUES[1], 'ja': DATE_VALUES[0],},),
    3: DateData(shared_date=DATE_VALUES[1],
                translated_date={'en': DATE_VALUES[0], 'ja': DATE_VALUES[2],},),
}

DATE_REVERSED = {
    DATE_VALUES[0]: DateData(shared_date=1, translated_date={'en': 3, 'ja': 2}),
    DATE_VALUES[1]: DateData(shared_date=3, translated_date={'en': 2, 'ja': 1}),
    DATE_VALUES[2]: DateData(shared_date=2, translated_date={'en': 1, 'ja': 3}),
}

#===============================================================================
